## Lawrence Chung Shing Hin
## CSCI 0931 Final Project
## Professor Anna Ritz

import re
import csv
import math
import decimal
import random
from random import choice

def wrapperFunction():

    # Read into WeeklyReturn.csv to get a list of stocks
    myFile = open('WeeklyReturn.csv','r')
    myString = myFile.read()
    myList = re.split('\n', myString)
    stocksList = re.split(',', myList[0])
    stocksList.remove('')
    
    # Step 1: remove dates from WeeklyStockPrice.csv
    removeDate('WeeklyStockPrice.csv','WeeklyStockPrice_NoDates.csv')

    # Step 2: calculate weekly stock returns and stock variance
    # (This step is done in Excel using the outfile from Step 1)

    # Step 3: Testing findVariance(Stock), findCorrelation(stockA, stockB),
    # and minPortfolioVariance(stockA, stockB)
    testFindVariance()
    testFindCorrelation()
    testMinPortfolioVariance()
    
    # Step 3: find out the variance-minimizing strategy for
    # 15 random stock pairs
    n = 1
    for n in range (0,10):
        stockA = choice(stocksList)
        stockB = choice(stocksList)
        if stockA == stockB:
            stockB = choice(stocksList)
        print 'When stock A is ' + stockA + ' and stock B is ' + stockB
        minPortfolioVariance(stockA, stockB)
        print ''

    return
    
def removeDate(infilename, outfile):
    ''' write a file that will take a CSV file of historical stock prices
    and outfile a CSV file without dates'''

    # Read into a CSV infile and turn it into a list 
    myFile = open(infilename, 'r')
    outFile = open(outfile, 'w')
    myString = myFile.read()
    myList = re.split('\n', myString)
    outputCells = ''

    # Step 1: use a for loop to write a string that contains
    # only one date in each row
    for line in myList:
        cells = re.split(',',line)
        outputCells += cells[0]
        for index in range (1,len(cells)):
            if index%2 == 1:
                outputCells += ',' + cells[index]
        outputCells += '\n'

    # Step 2: write the string in Step 1 into a CSV outfile 
    outFile.write(outputCells)
    return
        

def minPortfolioVariance(stockA,stockB):
    ''' find the mix of stock A and stock B that will minimize the variance
    of a two-stocks portfolio (consisting of stock A and stock B)'''

    # Step 1: find variance of Stock A and Stock B from CSV file
    varianceA = findVariance(stockA)
    varianceB = findVariance(stockB)

    # Step 2A: declare 3 variables-- portfolio variance, weight of Stock A,
    # and weight of Stock B 
    minPortfolioVariance = varianceB
    minVarWeightA = 0
    minVarWeightB = 100

    # Step 2B: use for range to find the weight of Stock A and Stock B that 
    # minimize the portfolio variance
    for weightA in range(5,100,5):
        portfolioVariance = calculatePortVar(stockA, stockB, weightA)
        ## print 'Investing ', weightA, '% in stock A and ', (100-weightA), '% in stock B will give a portfolio variance of ',portfolioVariance
        if portfolioVariance < minPortfolioVariance:
            minPortfolioVariance = portfolioVariance
            minVarWeightA = weightA
            minVarWeightB = 100 - weightA

    # Step 3: output the weight of Stock A and Stock B that minimize the
    # portfolio variance
    print 'A ', minVarWeightA, '% investment in Stock A and a ', minVarWeightB, '% investment in Stock B can minimize the porfolio variance.'

    return

def testMinPortfolioVariance():
    '''Test whether the for loop in minPortfolioVariance(stockA, stockB) works
    as expected'''

    print 'Testing minPortfolioVariance(stockA, stockB)......'

    # Step 1: simulate the for loop in minPortfolioVariance(stockA, stockB);
    # instead of calculating the portfolio variance, assign a random number to
    # portfolio variance in each scenario
    minPortfolioVariance = 100000
    variancesList = []
    for weightA in range(5,100,5):
        portfolioVariance = weightA*random.random()
        variancesList.append(portfolioVariance)
        if portfolioVariance < minPortfolioVariance:
            minPortfolioVariance = portfolioVariance

    # Step 2: if the for loop indeed runs through the 19 scenarios and returns
    # the smallest portfolio variance in all scenarios, then the for loop passes
    # the test
    if minPortfolioVariance == min(variancesList) and len(variancesList) == 19:
        print 'minPortfolioVariance(stockA, stockB) passes the test.'
        print 'The for-loop in minPortfolioVariance(stockA, stockB) works as expected in identifying the asset allocation with the smallest variance'

    # Step 3: otherwise the for loop fails the test
    else:
        print 'minPortfolioVarince(stockA, stockB) fails the test.'
        print 'Please go back and fix minPortfolioVariance(stockA, stockB)'
    
    print ''

    return
            
def findVariance(Stock):
    ''' match the variance of a stock from the WeeklyReturn.csv file'''
    
    # Read into a CSV file of weekly returns and stock variances
    myFile = open('WeeklyReturn.csv','r')
    myString = myFile.read()
    myList = re.split('\n', myString)
    
    # Step 1: Obtain a list of all stocks and a list stock variances
    stocksList = re.split(',', myList[0])
    variancesList = re.split(',',myList[len(myList)-2])

    # Step 2: Create a dictionary of all stocks and their corresponding variances
    varianceDict = {}
    for i in range (1,len(stocksList)):
        varianceDict[stocksList[i]] = variancesList[i]
                             
    # Step 3: Return error message if the input stock is not in the stcksList                         
    if Stock not in stocksList :
        print 'Please choose from the 20 specified stocks.'
        print 'The list of available stocks are ', stocksList
        return

    # Step 4: Find the variance for the input stock
    stockVariance = float(varianceDict[Stock])

    return stockVariance
            
def testFindVariance():
    ''' test whether findVariance(Stock) can return the corresponding variance
     for certain stocks'''

    print 'Testing findVariance(Stock)......'
    
    # Step 1: if findVariance(stock) returns the right variance for GOOG, CSCO
    # and QCOM, then findVariance(stock) passes the test.
    if (findVariance('GOOG') == 15.372 and findVariance('CSCO') == 16.743 and findVariance('QCOM') == 15.246):
        print 'Variance of GOOG, CSCO, and QCOM matches our data in Excel file' 
        print 'findVariance(Stock) passes the test.'

    # Step 2: otherwise, findVariance(stock) fails the test
    else:
        print 'Variance of GOOG, CSCO, and QCOM do not match our data in Excel file'
        print 'Variance of GOOG given by findVariance(\'GOOG\') is ', findVariance('GOOG')
        print 'Variance of CSCO given by findVariance(\'CSCO\') is ', findVariance('CSCO')
        print 'Variance of QCOM given by findVariance(\'QCOM\') is ', findVariance('QCOM')
        print 'Please go back and fix findVariance(Stock).'

    print''
    
    return

def calculatePortVar(stockA, stockB, weightA):
    ''' calculate the variance of a two-stock portfolio, given the variance
    of the two stocks and the investment in each stock '''

    # Step 1: find the variance of stock A and stock B
    varianceA = findVariance(stockA)
    varianceB = findVariance(stockB)

    # Step 2: find the correlation coefficient between stock A and stock B
    corr = findCorrelation(stockA, stockB)
    
    # Step 3: apply the formula to calculate the variance of sum of two variables    
    portfolioVariance = pow(weightA/100.0,2)*varianceA + pow(1.0-(weightA/100.0),2)*varianceB + 2*math.sqrt(varianceA)*math.sqrt(varianceB)*(weightA/100)*(1-(weightA/100))*corr
    
    return portfolioVariance

def findCorrelation(stockA, stockB):
    ''' match the correlation coefficient between stock A and stock B from
    CorrelationMatrix.csv'''

    # Read into a CSV file of Correlation Matrix between all stocks
    myFile = open('CorrelationMatrix.csv','r')
    myString = myFile.read()
    myList = re.split('\n', myString)

    # Step 1: Make my list into a list of lists by using a for range loop
    for i in range(0,len(myList)-1):
        myList[i] = re.split(',',myList[i])

    # Step 2: Identify the row and column that would give the desired
    # correlation cofficient
    stocksList = myList[0]
    if stockA not in stocksList:
        print 'Stock A is not chosen from the 20 available stocks.'
        print 'List of available stocks are', stocksList
        return
    if stockB not in stocksList:
        print 'Stock B is not chosen from the 20 available stocks.'
        print 'List of available stocks are', stocksList
        return
    numStockA = stocksList.index(stockA)
    numStockB = stocksList.index(stockB)

    # Step 3: Index into the list of lists to get the correlation cofficient
    correlation = float(myList[numStockA][numStockB])
    
    return correlation

def testFindCorrelation():
    ''' test whether findCorrelation returns the correlation coefficient
    of the two specified stocks'''

    print 'Testing findCorrelation(stockA, stockB)......'
    
    # Step 1: if findCorrelation(stockA, stockB) returns the right correlation
    # coefficient for GOOG and MSFT, CSCO and AMZN, BAC and QCOM, then
    # findCorrelation(stockA, stockB) passes the test
    if (findCorrelation('GOOG','MSFT') == 0.447 and findCorrelation('CSCO','AMZN') == 0.466 and findCorrelation('BAC','QCOM') == 0.244):
        print 'Correlation coefficient between GOOG and MSFT, CSCO and AMZN, BAC and QCOM match our data in the Excel file'
        print 'findCorrelation(stockA, stockB) passes the test.'

    # Step 2: otherwise, findCorrelation(stockA, stockB) fails the test 
    else:
        print 'Correlation coefficient between GOOG and MSFT, CSCO and AMZN, BAC and QCOM do not match our data in the Excel file'
        print 'Correlation given by findCorrelation(\'GOOG\',\'MSFT\') is', findCorrelation('GOOG', 'MSFT')
        print 'Correlation given by findCorrelation(\'CSCO\',\'AMZN\') is', findCorrelation('CSCO', 'AMZN')
        print 'Correlation given by findCorrelation(\'BAC\',\'QCOM\') is', findCorrelation('BAC', 'QCOM')
        print 'Please go back and fix findCorrelation(stockA, stockB).'

    print ''
    
    return
    
   

        
